% FullBNT path must be in Matlab paths

% variable/node numbering (NB: nodes must be topologically sorted)
FO = 1; BP = 2; LO = 3; DO = 4; HB = 5;

% variable names
names = {'fo','bp','lo','do','hb'};

% dag incidence matrix -- initialization
dag = zeros(5,5);

% dag incidence matrix -- the individual edges
% gives the network structure
dag(FO,LO)=1;
dag([FO BP],DO)=1;
dag(DO,HB) = 1;

% show the dag 
draw_graph(dag, names);

% determine the node sizes, i.e. the number of values they can take
% (family problem is binary -- all the variables are binary)
node_sizes = 2*ones(1,5);

% easily access toolbox help, an example call for mk_bnet function
doc mk_bnet
% initialize the bnet object
bnet = mk_bnet(dag, node_sizes, 'names', names);

% the network variables can be accessed through their names, e.g.:
FO = bnet.names('fo');

% specification of network parameters, conditional probability tables for the individual nodes
% the first value is regarded as T (true~1), the second as F (false~2) -- an arbitrary choice, needs top be kept in all the problem
% (BNT numbers the binary node values with indeces 1 and 2, the usual notation T~1, F~0 cannot be used)
bnet.CPD{FO} = tabular_CPD(bnet, FO, 'CPT', [0.15 0.85]');
bnet.CPD{BP} = tabular_CPD(bnet, BP, 'CPT', [0.01 0.99]');

% FO LO prob
% ----------
% T  T   0.6   P(LO=T|FO=T) = 0.6
% F  T   0.05  P(LO=T|FO=F) = 0.05
% T  F   0.4   P(LO=F|FO=T) = 0.4
% F  F   0.05  P(LO=F|FO=F) = 0.95
bnet.CPD{LO} = tabular_CPD(bnet, LO, 'CPT', [0.6 0.05 0.4 0.95]');

% FO BP DO prob
% --------------
% T  T  T  0.99   P(DO=T|FO=T,BP=T) = 0.99
% F  T  T  0.97   P(DO=T|FO=F,BP=T) = 0.97
% T  F  T  0.9    P(DO=T|FO=T,BP=F) = 0.9
% F  F  T  0.3    P(DO=T|FO=F,BP=F) = 0.3
% T  T  F  0.01   P(DO=F|FO=T,BP=T) = 0.01
% F  T  F  0.03   P(DO=F|FO=F,BP=T) = 0.03
% T  F  F  0.1    P(DO=F|FO=T,BP=F) = 0.1
% F  F  F  0.7    P(DO=F|FO=F,BP=F) = 0.7
bnet.CPD{DO} = tabular_CPD(bnet, DO, 'CPT', [0.99 0.97 0.9 0.3 0.01 0.03 0.1 0.7]');
bnet.CPD{HB} = tabular_CPD(bnet, HB, 'CPT', [0.7 0.01 0.3 0.99]');

% inference engine assignment
% uses variable elimination
engine = jtree_inf_engine(bnet); 
% other option is pearl ... uses belief propagation, iteration running informations stored into bel.txt file
% engine = pearl_inf_engine(bnet,'filename','bel.txt'); 

% evidence is represented in terms of node values, ie. T ~ 1, F ~ 2
% (see the CPT definitions above)
ev = cell(1,5);
ev{HB} = 2; % no barking heard ie. HB = F
ev{LO} = 1; % dorr light is off ie. LO = T
engine = enter_evidence(engine, ev);

% compute conditional probabilities
m = marginal_nodes(engine, FO);
fprintf('Pr(FO=T|HB=F,LO=T) = %5.4f\n', m.T(1))
fprintf('Pr(FO=F|HB=F,LO=T) = %5.4f\n', m.T(2))

% compute joint probabilities -- get their full table
% it cannot be done with each inference engine, 
% LW is a sampling engine -- approximate reasoning, accuracy can be increased when nsamples parameter is increased
engine = likelihood_weighting_inf_engine(bnet,'nsamples',1000);
ev = cell(1,5);
engine = enter_evidence(engine, ev);
m = marginal_nodes(engine, [FO BP LO DO HB]);
% Pr(FO=T,BP=F,LO=F,DO=T,HB=T) = 0.15*0.99*0.4*0.9*0.7=0.0374
fprintf('Pr(FO=T,BP=F,LO=F,DO=T,HB=T) = %5.4f\n', m.T(1,2,2,1,1))
% Pr(FO=F,BP=F,LO=F,DO=T,HB=T) = 0.85*0.99*0.95*0.3*0.7=0.1679
fprintf('Pr(FO=F,BP=F,LO=F,DO=T,HB=T) = %5.4f\n', m.T(2,2,2,1,1))

